@@ -25,6 +25,7 @@
 ;;; Code:
 
 (require 'mime-def)
+(require 'luna)
 (require 'std11)
 
 (autoload 'mime-entity-body-buffer "mime")
@@ -216,75 +217,89 @@ If is is not found, return DEFAULT-ENCODING."
 ;;; @ message parser
 ;;;
 
-(defun mime-parse-multipart (entity)
-  (with-current-buffer (mime-entity-body-buffer entity)
-    (let* ((representation-type
-	    (mime-entity-representation-type-internal entity))
-	   (content-type (mime-entity-content-type-internal entity))
-	   (dash-boundary
-	    (concat "--"
-		    (mime-content-type-parameter content-type "boundary")))
-	   (delimiter       (concat "\n" (regexp-quote dash-boundary)))
-	   (close-delimiter (concat delimiter "--[ \t]*$"))
-	   (rsep (concat delimiter "[ \t]*\n"))
-	   (dc-ctl
-	    (if (eq (mime-content-type-subtype content-type) 'digest)
-		(make-mime-content-type 'message 'rfc822)
-	      (make-mime-content-type 'text 'plain)
-	      ))
-	   (body-start (mime-entity-body-start-point entity))
-	   (body-end (mime-entity-body-end-point entity)))
-      (save-restriction
-	(goto-char body-end)
-	(narrow-to-region body-start
-			  (if (re-search-backward close-delimiter nil t)
-			      (match-beginning 0)
-			    body-end))
-	(goto-char body-start)
-	(if (re-search-forward
-	     (concat "^" (regexp-quote dash-boundary) "[ \t]*\n")
-	     nil t)
-	    (let ((cb (match-end 0))
-		  ce ncb ret children
-		  (node-id (mime-entity-node-id-internal entity))
-		  (i 0))
-	      (while (re-search-forward rsep nil t)
-		(setq ce (match-beginning 0))
-		(setq ncb (match-end 0))
-		(save-restriction
-		  (narrow-to-region cb ce)
-		  (setq ret (mime-parse-message representation-type dc-ctl
-						entity (cons i node-id)))
-		  )
-		(setq children (cons ret children))
-		(goto-char (setq cb ncb))
-		(setq i (1+ i))
-		)
-	      (setq ce (point-max))
-	      (save-restriction
-		(narrow-to-region cb ce)
-		(setq ret (mime-parse-message representation-type dc-ctl
-					      entity (cons i node-id)))
-		)
-	      (setq children (cons ret children))
-	      (mime-entity-set-children-internal entity (nreverse children))
-	      )
-	  (mime-entity-set-content-type-internal
-	   entity (make-mime-content-type 'message 'x-broken))
-	  nil)
-	))))
+;; (defun mime-parse-multipart (entity)
+;;   (with-current-buffer (mime-entity-body-buffer entity)
+;;     (let* ((representation-type
+;;             (mime-entity-representation-type-internal entity))
+;;            (content-type (mime-entity-content-type-internal entity))
+;;            (dash-boundary
+;;             (concat "--"
+;;                     (mime-content-type-parameter content-type "boundary")))
+;;            (delimiter       (concat "\n" (regexp-quote dash-boundary)))
+;;            (close-delimiter (concat delimiter "--[ \t]*$"))
+;;            (rsep (concat delimiter "[ \t]*\n"))
+;;            (dc-ctl
+;;             (if (eq (mime-content-type-subtype content-type) 'digest)
+;;                 (make-mime-content-type 'message 'rfc822)
+;;               (make-mime-content-type 'text 'plain)
+;;               ))
+;;            (body-start (mime-entity-body-start-point entity))
+;;            (body-end (mime-entity-body-end-point entity)))
+;;       (save-restriction
+;;         (goto-char body-end)
+;;         (narrow-to-region body-start
+;;                           (if (re-search-backward close-delimiter nil t)
+;;                               (match-beginning 0)
+;;                             body-end))
+;;         (goto-char body-start)
+;;         (if (re-search-forward
+;;              (concat "^" (regexp-quote dash-boundary) "[ \t]*\n")
+;;              nil t)
+;;             (let ((cb (match-end 0))
+;;                   ce ncb ret children
+;;                   (node-id (mime-entity-node-id-internal entity))
+;;                   (i 0))
+;;               (while (re-search-forward rsep nil t)
+;;                 (setq ce (match-beginning 0))
+;;                 (setq ncb (match-end 0))
+;;                 (save-restriction
+;;                   (narrow-to-region cb ce)
+;;                   (setq ret (mime-parse-message representation-type dc-ctl
+;;                                                 entity (cons i node-id)))
+;;                   )
+;;                 (setq children (cons ret children))
+;;                 (goto-char (setq cb ncb))
+;;                 (setq i (1+ i))
+;;                 )
+;;               (setq ce (point-max))
+;;               (save-restriction
+;;                 (narrow-to-region cb ce)
+;;                 (setq ret (mime-parse-message representation-type dc-ctl
+;;                                               entity (cons i node-id)))
+;;                 )
+;;               (setq children (cons ret children))
+;;               (mime-entity-set-children-internal entity (nreverse children))
+;;               )
+;;           (mime-entity-set-content-type-internal
+;;            entity (make-mime-content-type 'message 'x-broken))
+;;           nil)
+;;         ))))
 
-(defun mime-parse-encapsulated (entity)
-  (mime-entity-set-children-internal
-   entity
-   (with-current-buffer (mime-entity-body-buffer entity)
-     (save-restriction
-       (narrow-to-region (mime-entity-body-start-point entity)
-			 (mime-entity-body-end-point entity))
-       (list (mime-parse-message
-	      (mime-entity-representation-type-internal entity) nil
-	      entity (cons 0 (mime-entity-node-id-internal entity))))
-       ))))
+;; (defun mime-parse-encapsulated (entity)
+;;   (mime-entity-set-children-internal
+;;    entity
+;;    (with-current-buffer (mime-entity-body-buffer entity)
+;;      (save-restriction
+;;        (narrow-to-region (mime-entity-body-start-point entity)
+;;                          (mime-entity-body-end-point entity))
+;;        (list (mime-parse-message
+;;               (mime-entity-representation-type-internal entity) nil
+;;               entity (cons 0 (mime-entity-node-id-internal entity))))
+;;        ))))
+
+;; (defun mime-parse-external (entity)
+;;   (require 'mmexternal)
+;;   (mime-entity-set-children-internal
+;;    entity
+;;    (with-current-buffer (mime-entity-body-buffer entity)
+;;      (save-restriction
+;;        (narrow-to-region (mime-entity-body-start-point entity)
+;;                          (mime-entity-body-end-point entity))
+;;        (list (mime-parse-message
+;;               'mime-external-entity nil
+;;               entity (cons 0 (mime-entity-node-id-internal entity))))
+;;        ;; [tomo] Should we unify with `mime-parse-encapsulated'?
+;;        ))))
 
 (defun mime-parse-message (representation-type &optional default-ctl 
 					       parent node-id)
@@ -331,10 +346,8 @@ If is is not found, return DEFAULT-ENCODING."
 If buffer is omitted, it parses current-buffer."
   (save-excursion
     (if buffer (set-buffer buffer))
-    (setq mime-message-structure
-	  (mime-parse-message (or representation-type
-				  'mime-buffer-entity) nil))
-    ))
+    (mime-parse-message (or representation-type
+			    'mime-buffer-entity) nil)))
 
 
 ;;; @ end
